proc layer_makeframe {} {

   global g_ddhlayer_usedata
   global g_ddhlayer_autorefresh

   # NUMBER OF LAYERS
   # Must be between 2 and 10, inclusive
   set num_layers 10

   # DEFAULT "SAVE LAYER INFO" TO UNCHECKED
   # Setting this variable to 0 will cause the "Save layer info in AC3D data" checkbox to
   # be enabled by default.
   set g_ddhlayer_usedata 1

   # DEFAULT "AUTOREFRESH" TO CHECKED
   # Setting this variable to 1 will cause the "Use auto-refresh" checkbox to
   # be enabled by default.
   set g_ddhlayer_autorefresh 1

   # ENABLE SHORT LABELS
   # Set the variable "ddhlayer_enable_shortlabels" (below) to 1 to enable abbreviated
   # button labels.  This simply takes less space in the GUI.
   set ddhlayer_enable_shortlabels 1

   if { $ddhlayer_enable_shortlabels == 1 } {
      set ddhlayer_label_add      "+"
      set ddhlayer_label_show     "S"
      set ddhlayer_label_hide     "H"
      set ddhlayer_label_select   "s"
      set ddhlayer_label_unselect "d"
   } else {
      set ddhlayer_label_add      "add"
      set ddhlayer_label_show     "show"
      set ddhlayer_label_hide     "hide"
      set ddhlayer_label_select   "sel"
      set ddhlayer_label_unselect "desel"
   }

   toplevel .layer
   wm title .layer "Layers"
   wm transient .layer [winfo toplevel [winfo parent .layer]]

   set width 6
   set ypad 3

   set ddhlayer_enable_showhide 1

   global g_ddhlayer_settingfile
   set g_ddhlayer_settingfile "null"

   for { set ix 0 } { $ix < $num_layers } { incr ix } {
      #global g_ddhlayer_count$ix
      #set g_ddhlayer_count$ix 0
      # Frame
      frame .layer.frm$ix
      pack .layer.frm$ix -side top -fill x -pady $ypad
      # Layer #
      label .layer.frm$ix.lbl$ix -text "$ix: "
      pack .layer.frm$ix.lbl$ix -side left -fill x -expand 1
      add_balloon .layer.frm$ix.lbl$ix "Layer number"
      # Layer object count
      #label .layer.frm$ix.txtobjs$ix -textvariable g_ddhlayer_count$ix -width 4 -justify right
      label .layer.frm$ix.txtobjs$ix -text 0 -width 4 -justify right
      #entry .layer.frm$ix.txtobjs$ix -textvariable g_ddhlayer_count$ix -width 4 -justify right
      pack .layer.frm$ix.txtobjs$ix -side left -fill x -expand 1
      add_balloon .layer.frm$ix.txtobjs$ix "Number of objects in this layer"
      # Add/move objects to layer
      button .layer.frm$ix.btnmove$ix -text $ddhlayer_label_add -command "eval ac3d ddhlayer_move $ix"
      pack .layer.frm$ix.btnmove$ix -side left -fill x -expand 1 -padx 4
      add_balloon .layer.frm$ix.btnmove$ix "Add/move selected objects to this layer"
      # Only display show/hide buttons if user wants them
      if { $ddhlayer_enable_showhide == 1 } {
         # Show objects
         button .layer.frm$ix.btnshow$ix -text $ddhlayer_label_show -command "eval ac3d ddhlayer_show $ix"
         pack .layer.frm$ix.btnshow$ix -side left -fill x -expand 1
         add_balloon .layer.frm$ix.btnshow$ix "Show (unhide) all objects in this layer"
         # Hide objects
         button .layer.frm$ix.btnhide$ix -text $ddhlayer_label_hide -command "eval ac3d ddhlayer_hide $ix"
         pack .layer.frm$ix.btnhide$ix -side left -fill x -expand 1
         add_balloon .layer.frm$ix.btnhide$ix "Hide all objects in this layer"
      }
      # Select visible objects
      button .layer.frm$ix.btnsel$ix -text $ddhlayer_label_select -command "eval ac3d ddhlayer_select $ix"
      pack .layer.frm$ix.btnsel$ix -side left -fill x -expand 1
      add_balloon .layer.frm$ix.btnsel$ix "Select all objects in this layer"
      # Deselect objects
      button .layer.frm$ix.btndesel$ix -text $ddhlayer_label_unselect -command "eval ac3d ddhlayer_deselect $ix"
      pack .layer.frm$ix.btndesel$ix -side left -fill x -expand 1
      add_balloon .layer.frm$ix.btndesel$ix "Deselect all objects in this layer"
   }

   frame .layer.frmusedata
   pack .layer.frmusedata -side bottom -fill x -pady $ypad
   # Use data indicator
   checkbutton .layer.frmusedata.btnusedata -text "Save layer info in AC3D data" -variable g_ddhlayer_usedata -command {
      eval ac3d ddhlayer_usedata $g_ddhlayer_usedata
   }
   pack .layer.frmusedata.btnusedata -side top -fill x -expand 1
   add_balloon .layer.frmusedata.btnusedata "Use each objects' Data component to store layer info\nAllows layer data to be saved in AC3D files"
   # Use the auto-refresh callback
   checkbutton .layer.frmusedata.btnautorefresh -text "Autorefresh" -variable g_ddhlayer_autorefresh -command {
      if { $g_ddhlayer_autorefresh == 1 } {
         eval ac3d ddhlayer_refreshlayerlist
         eval ac3d ddhlayer_register_callback
      } else {
         eval ac3d ddhlayer_unregister_callback
      }
   }
   pack .layer.frmusedata.btnautorefresh -side top -fill x -expand 1
   add_balloon .layer.frmusedata.btnautorefresh "Automatically refresh layer data.\nReads layer data from AC3D files on Open/Import"
   # Refresh layer from object data
   button .layer.frmusedata.btnrefresh -text "Refresh" -command {
      eval ac3d ddhlayer_refreshlayerlist
   }
   pack .layer.frmusedata.btnrefresh -side bottom -fill x -expand 1
   add_balloon .layer.frmusedata.btnrefresh "Refresh layer info - only needed if Autorefresh is disabled"

   wm protocol .layer WM_DELETE_WINDOW {
      eval ac3d ddhlayer_clear {$g_ddhlayer_settingfile} $g_ddhlayer_usedata
      destroy .layer
   }

   eval ac3d ddhlayer_usedata $g_ddhlayer_usedata
   eval ac3d ddhlayer_refreshlayerlist
   eval ac3d ddhlayer_register_callback

}

proc layer_dialog {} {

   if ![winfo exists .layer] {
      layer_makeframe
   }

   wm deiconify .layer
   centre_window_on_screen .layer

}

